import {
  MatcherException,
  MatcherExceptionCode,
} from 'src/modules/matcher/exceptions/matcher.exception';
import { MatchQuery } from '../../../queries/match.query';
import { Algorithm } from '../../types/algorithm.enum';
import { Match } from '../ecosystem/match';
import { Candidate } from './candidate';
import { AlgorithmFactory } from './factory/algorithm-factory.abstract';
import { ClassicAlgorithmFactory } from './factory/classic';

export class Matcher {
  match = (matchQuery: MatchQuery): Array<Match> => {
    let algorithm: AlgorithmFactory;
    switch (matchQuery.algorithmSettings.algorithm) {
      case Algorithm.CLASSIC:
        algorithm = new ClassicAlgorithmFactory(matchQuery);
        break;
      default:
        throw new MatcherException(
          MatcherExceptionCode.INVALID_ARGUMENT,
          'Unknown algorithm',
        );
    }
    let candidates: Array<Candidate> = [];
    for (const processor of algorithm.createProcessors()) {
      candidates = processor.execute(candidates);
    }
    return [];
  };
}
